package RT::Action::ExtractCustomFieldValues;
require RT::Action::Generic;

use strict;
use vars qw/@ISA/;
@ISA=qw(RT::Action::Generic);

sub Describe  {
  my $self = shift;
  return (ref $self );
}

sub Prepare {
  return (1);
}

sub Commit {
  my $self = shift;
  my $Ticket = $self->TicketObj;
  my $Transaction = $self->TransactionObj;
  my $Content = $self->TemplateObj->Content;
  my $Queue = $Ticket->QueueObj->Id;

  # go through each line of the template's content to perform scans
  my @lines = split(/[\n\r]+/,$Content);
  for (@lines) {
    chomp;
    # ignore comments and empty lines
    next if (/^#/);
    next if (/^\s*$/);
    # split the line into its compnents, using | as the separator
    my ($CustomFieldName,$InspectField,$MatchString,$PostEdit) = split(/\|/);
    # try to load the Custom Field, 1st as belonging to the queue, 2nd global
    $RT::Logger->debug("load cf $CustomFieldName");
    my $cf = new RT::CustomField($RT::SystemUser);
    my ($id,$msg) = $cf->LoadByNameAndQueue (Name=>"$CustomFieldName", Queue=>$Queue);
    if (! $id) {
      ($id,$msg) = $cf->LoadByNameAndQueue (Name=>"$CustomFieldName", Queue=>0);
    }
    $RT::Logger->debug("load cf done: $id $msg");
    # only go further if Custom Field is found
    if ($id) {
      my $found = 0;
      # scan body or header field, putting match into $_
      if ($InspectField =~ /^body$/i) {
        $RT::Logger->debug("look for cf in Body");
        $found = ($Transaction->Attachments->First->Content =~ /$MatchString/m);
        if ($1) { $_ = $1; } else { $_ = $&; }
      } else {
        $RT::Logger->debug("look for cf in Header $InspectField");
        $found = ($Transaction->Attachments->First->GetHeader("$InspectField") =~ /$MatchString/);
        if ($1) { $_ = $1; } else { $_ = $&; }
      }
      # be sure the match is in $_
      if ($found) {
        $RT::Logger->debug("matched value: $_");
      } else {
        $_ = "";
      }
      # if there is a PostEdit-Entry in the scanner row, evaluate it 
      if ($_ && $PostEdit) {
        eval ($PostEdit);
        $RT::Logger->debug("transformed ($PostEdit) value: $_");
      }
      if ($_) {
        $RT::Logger->debug("found value for cf: $_");
        ($id,$msg) = $Ticket->AddCustomFieldValue (Field => $cf, Value => $_);
        $RT::Logger->info("CustomFieldValue ($CustomFieldName,$_) added: $id $msg");
      }
    }
  }
  return(1);
}

1;
